// Human.cpp: implementation of the CHuman class
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MaxFight.h"
#include "Human.h"

#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

int NSpeedMult= 1;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHuman::CHuman()
{
	m_pBody= &m_Body[0];
	m_pNeck= &m_Body[1];
	m_pHead= &m_Body[2];
	
	m_pArmRight= &m_Body[3];
	m_pHandRight= &m_Body[4];
	m_pFistRight= &m_Body[5];

	m_pArmLeft= &m_Body[6];
	m_pHandLeft= &m_Body[7];
	m_FistLeft= &m_Body[8];

	m_LegRight= &m_Body[9];
	m_Leg2Right= &m_Body[10];
	m_FootRight= &m_Body[11];

	m_LegLeft= &m_Body[12];
	m_Leg2Left= &m_Body[13];
	m_FootLeft= &m_Body[14];

	m_pBody->SetParams(NULL, 0, 90, -90, 100, 14, 0, -1);
	m_pNeck->SetParams(m_pBody, 3, 0, 0, 6, 5, 0, -1);
	m_pHead->SetParams(m_pNeck, 3, 20, -20, 20, 12, 0, -1);

	m_pArmRight->SetParams(m_pBody, 4, 90, -180, 50, 6, -25);
	m_pHandRight->SetParams(m_pArmRight, 3, 0, -170, 50, 6, -90);
	m_pFistRight->SetParams(m_pHandRight, 3, 0, 0, 5, 6, 0);

	m_pArmLeft->SetParams(m_pBody, 2, 90, -180, 50, 6, 5);
	m_pHandLeft->SetParams(m_pArmLeft, 3, 0, -170, 50, 6, -100);
	m_FistLeft->SetParams(m_pHandLeft, 3, 0, 0, 5, 6, 0);

	m_LegRight->SetParams(m_pBody, 5, 160, -160, 60, 7, -5);
	m_Leg2Right->SetParams(m_LegRight, 3, 170, 0, 60, 7, 0);
	m_FootRight->SetParams(m_Leg2Right, 3, -90, -90, 10, 7, -90);

	m_LegLeft->SetParams(m_pBody, 1, 160, -160, 60, 7, 5);
	m_Leg2Left->SetParams(m_LegLeft, 3, 170, 0, 60, 7, 0);
	m_FootLeft->SetParams(m_Leg2Left, 3, -90, -90, 10, 7, -90);

	m_nCurPart= 1000; // no part
	m_bActive= false;
	m_bEnemy= false;

	ClearPartsMoved();
}

CHuman::~CHuman()
{
}
//------------------------------------------------------

void CPartBody::SetParams(CPartBody* pParent, int indParent, int AngPos, int AngNeg, int nHeight, int nWidth, int nCurAng, int nMult)
{
	m_ParentPart= pParent;
	m_indParent= indParent;
	m_AngPos= AngPos;
	m_AngNeg= AngNeg;
	m_CurAng= nCurAng;

	m_Coords[0].x= 0; // middle point on bottom
	m_Coords[0].y= 0;

	m_Coords[1].x= nWidth/2 *nMult;
	m_Coords[1].y= nHeight/20 *nMult;

	m_Coords[2].x= nWidth/2 *nMult;
	m_Coords[2].y= (nHeight - nHeight/20) *nMult;

	m_Coords[3].x= 0; // middle point on top
	m_Coords[3].y= nHeight *nMult;

	m_Coords[4].x= -nWidth/2 *nMult;
	m_Coords[4].y= (nHeight - nHeight/20) *nMult;

	m_Coords[5].x= -nWidth/2 *nMult;
	m_Coords[5].y= nHeight/20 *nMult;
}
//------------------------------------------------------

void CHuman::DrawHuman(CDC* pDC)
{
	CPen Pen, *pOldPen= NULL;
	CBrush Brush, BrushSel, *pOldBr= NULL;

	Pen.CreatePen(PS_SOLID, 1, RGB(10, 10, 10) );
	if(m_Body[0].m_Coords[1].x > 0){
		// Enemy
		if(m_bActive)
			Brush.CreateSolidBrush(RGB(230, 160, 160) );
		else
			Brush.CreateSolidBrush(RGB(180, 160, 160) );
	}
	else{
		// Player
		if(m_bActive) // moving
			Brush.CreateSolidBrush(RGB(0, 140, 140) );
		else
			Brush.CreateSolidBrush(RGB(130, 150, 150) );
	}

	BrushSel.CreateSolidBrush(RGB(200, 150, 150) );
	pOldPen= pDC->SelectObject(&Pen);
	pOldBr= pDC->SelectObject(&Brush);

	int p, AngPar;
	CPoint Point, pointPar(0,0), point0(0,0);
	CPartBody *pParent, *pParent1;
	CArray <CPoint, CPoint> PointsArray;
	PointsArray.SetSize(NUM_POINTS+1);

	// draw Man1 from end part, and Man2 form first part
	int i= NUM_PARTS-1, iInc= -1, iEnd= 0;
	if(m_Body[0].m_Coords[1].x > 0)
		i= 0, iInc= 1, iEnd= NUM_PARTS-1;

	while(true)
	{
		// get parent coords
		pointPar.x= m_Position.x, pointPar.y= m_Position.y;
		AngPar= 0;

		if(i == m_nCurPart)
			pDC->SelectObject(&BrushSel);

		pParent= m_Body[i].m_ParentPart;
		if(pParent){ // except Body
			// pass parent parts - only 3 cycle
			pParent1= pParent->m_ParentPart;
			if(pParent1){
				if(pParent1->m_ParentPart){
					AngPar += pParent1->m_ParentPart->m_CurAng;
					Point= pParent1->m_ParentPart->m_Coords[pParent1->m_indParent];
					Point= RotatePoint(Point, point0, AngPar, 0,0);
					pointPar += Point;
				}
				AngPar += pParent1->m_CurAng;
				Point= pParent1->m_Coords[pParent->m_indParent];
				Point= RotatePoint(Point, point0, AngPar, 0,0);
				pointPar += Point;
			}

			AngPar += pParent->m_CurAng;
			Point= pParent->m_Coords[m_Body[i].m_indParent];
			Point= RotatePoint(Point, point0, AngPar, 0,0);
			pointPar += Point;
		}
		
		PointsArray[0]= pointPar;
		for(p= 0; p < NUM_POINTS; p++){
			// rotate point
			Point= RotatePoint(m_Body[i].m_Coords[p], point0, AngPar + m_Body[i].m_CurAng, 0,0);
			Point += pointPar;

			PointsArray[p]= Point;

			m_Body[i].m_WinCoords[p]= Point;
		}
		PointsArray[p]= pointPar;
		pDC->Polygon(PointsArray.GetData(), NUM_POINTS+1);

		if(i == m_nCurPart)
			pDC->SelectObject(&Brush);
		
		i += iInc;
		if(iInc > 0){
			if(i > iEnd)
				break;
		}
		else if(i < iEnd)
			break;
	}

	if(pOldPen)
		pDC->SelectObject(pOldPen);
	if(pOldBr)
		pDC->SelectObject(pOldBr);
}
//------------------------------------------------------

// Execute move for nPart part of body, nMove - type of move
// Change position of part step by step
// Returns true - move is finished
//			false - move is not finished
bool CHuman::MakeMove(int nPart, int nMove, CHuman& Man2, int xPosBeg)
{
	int nPart1= nPart/3 * 3; // leg or arm
	bool bFinished= true;
	
	int nPositStep= 50;
	int nAng90= -90, nPositAdd= 1, nAng5= -5, nAng160= -160;
	if(m_Body[0].m_Coords[1].x > 0)
		nAng90= 90, nPositAdd= -1, nAng5= 5, nAng160= 160;

	if(nPart < 3){
		// it's a Body
		m_Body[nPart1].m_CurAng= 0;
		switch(nMove) 
		{ 
		case opAttack:
			// move body forward on nPositAdd
			m_Position.x += nPositAdd;
			if(m_Position.x < nPositStep || m_Position.x > 550)
				m_Position.x -= nPositAdd;
			else if( abs(m_Position.x - xPosBeg) < nPositStep )
				bFinished= false;
			break;

		case opBack:
			// move body backward
			m_Position.x -= nPositAdd;
			if(m_Position.x < nPositStep || m_Position.x > 550)
				m_Position.x += nPositAdd;
			else if( abs(m_Position.x - xPosBeg) < nPositStep )
				bFinished= false;
			break;

		case opReturn:
			// original position
			SetBodyOrig();
			break;

		case opDown:
			// move body down
			// Make Leg = max pos angle
			nPart1= 9;
			if(m_Body[nPart1].m_CurAng < nAng160)
				m_Body[nPart1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1].m_CurAng > nAng160)
				m_Body[nPart1].m_CurAng--, bFinished= false;
			// Make Leg2 = max neg angle
			if(m_Body[nPart1+1].m_CurAng < -nAng160)
				m_Body[nPart1+1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1+1].m_CurAng > -nAng160)
				m_Body[nPart1+1].m_CurAng--, bFinished= false;
			
			nPart1= 12;
			if(m_Body[nPart1].m_CurAng < nAng160)
				m_Body[nPart1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1].m_CurAng > nAng160)
				m_Body[nPart1].m_CurAng--, bFinished= false;
			// Make Leg2 = max neg angle
			if(m_Body[nPart1+1].m_CurAng < -nAng160)
				m_Body[nPart1+1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1+1].m_CurAng > -nAng160)
				m_Body[nPart1+1].m_CurAng--, bFinished= false;
			
			if(m_Position.y < VERT_POS + 100) // 100 - down height
				m_Position.y++;
			break;

		case opUp:
			// Make Leg = 0 degrees
			nPart1= 9;
			if(m_Body[nPart1].m_CurAng < nAng5)
				m_Body[nPart1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1].m_CurAng > nAng5)
				m_Body[nPart1].m_CurAng--, bFinished= false;
			// Make Leg2 = 0 degrees
			if(m_Body[nPart1+1].m_CurAng < 0)
				m_Body[nPart1+1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1+1].m_CurAng > 0)
				m_Body[nPart1+1].m_CurAng--, bFinished= false;

			nPart1= 12;
			if(m_Body[nPart1].m_CurAng < -nAng5)
				m_Body[nPart1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1].m_CurAng > -nAng5)
				m_Body[nPart1].m_CurAng--, bFinished= false;
			// Make Leg2 = 0 degrees
			if(m_Body[nPart1+1].m_CurAng < 0)
				m_Body[nPart1+1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1+1].m_CurAng > 0)
				m_Body[nPart1+1].m_CurAng--, bFinished= false;

			if(m_Position.y > VERT_POS)
				m_Position.y--;
			break;
		}
	}
	else{
		// it's hand or leg
		switch(nMove) 
		{ 
		case opAttack:
			if(nPart1 == 9 && abs(m_Body[nPart1+3].m_CurAng) > abs(nAng90/4) )
				// don't move second leg if 1-st is up
				break;
			if(nPart1 == 12 && abs(m_Body[nPart1-3].m_CurAng) > abs(nAng90/4) )
				// don't move second leg if 1-st is up
				break;

			// Make Arm or Leg = 90 degrees
			if(m_Body[nPart1].m_CurAng < nAng90)
				m_Body[nPart1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1].m_CurAng > nAng90)
				m_Body[nPart1].m_CurAng--, bFinished= false;
			
			// Make Hand or Leg2 = 0 degrees
			if(m_Body[nPart1+1].m_CurAng < 0)
				m_Body[nPart1+1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1+1].m_CurAng > 0)
				m_Body[nPart1+1].m_CurAng--, bFinished= false;
			break;
			
		case opBack:
			// Make Arm or Leg = 0 degrees
			if(m_Body[nPart1].m_CurAng < -nAng5)
				m_Body[nPart1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1].m_CurAng > -nAng5)
				m_Body[nPart1].m_CurAng--, bFinished= false;
			
			// Make Hand or Leg2 = 0 degrees
			if(m_Body[nPart1+1].m_CurAng < 0)
				m_Body[nPart1+1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1+1].m_CurAng > 0)
				m_Body[nPart1+1].m_CurAng--, bFinished= false;
			break;
			
		case opBlock:
			// Make Arm or Leg = 90 degrees
			if(m_Body[nPart1].m_CurAng < nAng90)
				m_Body[nPart1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1].m_CurAng > nAng90)
				m_Body[nPart1].m_CurAng--, bFinished= false;
			
			if(nPart1 == 9 || nPart1 == 12)
				// Bend leg to another side
				nAng90= -nAng90;

			// Make Hand or Leg2 = 0 degrees
			if(m_Body[nPart1+1].m_CurAng < nAng90)
				m_Body[nPart1+1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1+1].m_CurAng > nAng90)
				m_Body[nPart1+1].m_CurAng--, bFinished= false;
			break;
			
		case opUp:
			if(nPart1 == 9 && abs(m_Body[nPart1+3].m_CurAng) > abs(nAng90/2) )
				// don't move second leg if 1-st is up
				break;
			if(nPart1 == 12 && abs(m_Body[nPart1-3].m_CurAng) > abs(nAng90/2) )
				// don't move second leg if 1-st is up
				break;

			// Make Arm or Leg = 90 degrees
			if(m_Body[nPart1].m_CurAng < nAng90*2)
				m_Body[nPart1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1].m_CurAng > nAng90*2)
				m_Body[nPart1].m_CurAng--, bFinished= false;
			
			// Make Hand or Leg2 = 0 degrees
			if(m_Body[nPart1+1].m_CurAng < 0)
				m_Body[nPart1+1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1+1].m_CurAng > 0)
				m_Body[nPart1+1].m_CurAng--, bFinished= false;
			break;
			
		case opDown:
			// Make Arm or Leg = 90 degrees
			if(m_Body[nPart1].m_CurAng < 0)
				m_Body[nPart1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1].m_CurAng > 0)
				m_Body[nPart1].m_CurAng--, bFinished= false;
			
			// Make Hand or Leg2 = 0 degrees
			if(m_Body[nPart1+1].m_CurAng < 0)
				m_Body[nPart1+1].m_CurAng++, bFinished= false;
			else if(m_Body[nPart1+1].m_CurAng > 0)
				m_Body[nPart1+1].m_CurAng--, bFinished= false;
			break;

		case opReturn:
			// original position
			SetBodyOrig();
			break;
		}
	}
	Sleep(10 * NSpeedMult);
	return bFinished;
}
//------------------------------------------------------

void CHuman::AddAngle(int nAdd)
{
	if(m_nCurPart < NUM_PARTS){
		int nAngPrev= m_Body[m_nCurPart].m_CurAng;
		if(m_Body[m_nCurPart].m_Coords[3].y < 0)
			m_Body[m_nCurPart].m_CurAng -= nAdd;
		else
			m_Body[m_nCurPart].m_CurAng += nAdd;

		if( m_Body[m_nCurPart].m_CurAng > m_Body[m_nCurPart].m_AngPos)
			m_Body[m_nCurPart].m_CurAng= m_Body[m_nCurPart].m_AngPos;
		if( m_Body[m_nCurPart].m_CurAng < m_Body[m_nCurPart].m_AngNeg)
			m_Body[m_nCurPart].m_CurAng= m_Body[m_nCurPart].m_AngNeg;

		if(m_nCurPart == 0){
			// it's a Body - rotate also legs
			int nAngDiff= m_Body[m_nCurPart].m_CurAng - nAngPrev;
			m_LegLeft->m_CurAng -= nAngDiff;
			m_LegRight->m_CurAng -= nAngDiff;
		}
	}
}
//------------------------------------------------------

void CHuman::InvertBody()
{
	int i, p, nAng;
	// invert horizontal body coords
	for(p= 0; p < NUM_POINTS; p++)
		m_Body[0].m_Coords[p].x= -m_Body[0].m_Coords[p].x;
	
	// invert angles of body parts
	for(i= 0; i < NUM_PARTS; i++){
		m_Body[i].m_CurAng= -m_Body[i].m_CurAng;
		nAng= m_Body[i].m_AngPos;
		m_Body[i].m_AngPos= -m_Body[i].m_AngNeg;
		m_Body[i].m_AngNeg= -nAng;
	}
}
//------------------------------------------------------

bool CHuman::IsPartMoved(int nPart, int nMove)
{
	int nPart1= nPart/3;

	if( nPart%3 == 0){ // it's first part of 3 - means right and left
		if(nPart1 == 2) nPart1= 1; // set 1-st arm
		if(nPart1 == 4) nPart1= 3; // set 1-st leg
		
		if(m_nPartsMoved[nPart1] == nMove || m_nPartsMoved[nPart1+1] == nMove)
			// test Left and Right
			return true;
		else return false;
	}
	
	return m_nPartsMoved[nPart1] == nMove; 
}
//------------------------------------------------------

void CHuman::SetBodyOrig()
{
	for(int i= 0; i < NUM_PARTS-1; i++)
		if(i != 11 )
			// don't move feet
			m_Body[i].m_CurAng= 0;
	
	int nAng5= -5;
	if(m_Body[0].m_Coords[1].x > 0)
		nAng5= 5;

	m_pArmRight->m_CurAng= nAng5;
	m_pArmLeft->m_CurAng= -nAng5;
	m_LegRight->m_CurAng= nAng5;
	m_LegLeft->m_CurAng= -nAng5;
	
	m_Position.y= VERT_POS;
}
//------------------------------------------------------

void CPartBody::GetRegion(CRgn &Rgn)
{
	CArray <CPoint, CPoint> PointsArray;
	PointsArray.SetSize(NUM_POINTS+1);

	for(int p= 0; p < NUM_POINTS; p++)
		PointsArray[p]= m_WinCoords[p];
	PointsArray[p]= m_WinCoords[0];

	Rgn.CreatePolygonRgn(PointsArray.GetData(), NUM_POINTS+1, ALTERNATE);
}
//------------------------------------------------------

// Rotate point in relation to point0
CPoint RotatePoint(const CPoint &point, const CPoint &point0, double ang, int a, int b)
{
	CPoint res;
   	// to radians
	ang= ang * atan(1.0) / 45;
	res.x= (int)floor(point0.x+(point.x-point0.x)*cos(ang)-(point.y-point0.y)*sin(ang)+a+0.5);
	res.y= (int)floor(point0.y+(point.x-point0.x)*sin(ang)+(point.y-point0.y)*cos(ang)+b+0.5);

	return res;
}

// distance between two 3D points
double DistPoints(CPoint &P1, CPoint &P2)
{
	return sqrt( pow(P1.x - P2.x, 2) + pow(P1.y - P2.y, 2) );
}
//------------------------------------------------------
