#if !defined(AFX_CHUMAN_H_INCLUDED_)
#define AFX_CHUMAN_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define NUM_PARTS 15
#define NUM_POINTS 6

class CPartBody
{
	// coords of Part
	CPoint m_Coords[NUM_POINTS];

	// coords of m_Coords point on screen
	CPoint m_WinCoords[NUM_POINTS];

	// current angle of part related to it's parent
	int m_CurAng;
	// possible rotate angles - positive and negative
	int m_AngPos, m_AngNeg;

	// part of body to which this part is connected - Parent part, NULL - no parent
	CPartBody* m_ParentPart;
	int m_indParent;

public:
	CPartBody(){ m_ParentPart= NULL; m_indParent= 0; m_CurAng= m_AngPos= m_AngNeg= 0; } // m_AngPos = 0 - no move
	virtual ~CPartBody(){ };

	void SetParams(CPartBody* pParent, int indParent, int AngPos, int AngNeg, int nHeight, int nWidth, int nCurAng, int nMult= 1);
	void GetRegion(CRgn &Rgn);

	friend class CHuman;
	friend class CProgramData;
};

//----------------------------
// Main class of Human

enum { opAttack= 0, opBack, opBlock, opUp, opDown, opReturn, opIf };
const int VERT_POS= 150;

class CHuman
{
	// Body parts
	CPartBody m_Body[NUM_PARTS];
	// Array of last Moves for all Parts
	int m_nPartsMoved[NUM_PARTS/3];

	CPoint m_Position;
	int m_nCurPart;
	bool m_bActive; // is Human moving

	CPartBody *m_pBody, *m_pNeck, *m_pHead,
			*m_pArmRight, *m_pHandRight, *m_pFistRight,
			*m_pArmLeft, *m_pHandLeft, *m_FistLeft, 
			*m_LegRight, *m_Leg2Right, *m_FootRight,
			*m_LegLeft, *m_Leg2Left, *m_FootLeft;

	friend class CProgramData;

public:
	bool m_bEnemy; // is Enemy

	CHuman();
	virtual ~CHuman();

	void DrawHuman(CDC* pDC);
	CPartBody* GetPart(int nPart){ if(nPart < NUM_PARTS) return &m_Body[nPart]; else return NULL; }
	void SetCurrentPart(int nCurPart){ m_nCurPart= nCurPart; }
	void AddAngle(int nAdd);
	void SetPosition(CPoint pos){ m_Position= pos; }
	CPoint GetPosition(){ return m_Position; }
	void InvertBody();
	void SetBodyOrig();

	bool MakeMove(int nPart, int nMove, CHuman& Man2, int xPosBeg);
	bool IsPartMoved(int nPart, int nMove);
	void SetPartMoved(int nPart, int nMove){ m_nPartsMoved[ nPart/3 ]= nMove; }
	void ClearPartsMoved(){ for(int i= 0; i < NUM_PARTS/3; i++) m_nPartsMoved[i]= -1; }
	void SetActive(bool bActive= true){ m_bActive= bActive; }
};

extern CHuman Human, Enemy;

extern int NSpeedMult; // speed of moving of Human

CPoint RotatePoint(const CPoint &point, const CPoint &point0, double ang, int a, int b);

#endif // !defined(AFX_CHUMAN_H_INCLUDED_)
